package uk.co.his.test.jaxb.junit;

import java.io.File;
import java.io.IOException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.junit.Test;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import uk.co.his.test.jaxb.model.Child;
import uk.co.his.test.jaxb.model.DummyEnvelope;
import uk.co.his.test.jaxb.model.Leaf;
import uk.co.his.test.jaxb.model.Root;

public class TestMoxy {
	
	public static final File test1 = new File("test-output/Test1.xml");

	@Test
	public void test1() throws JAXBException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext1();
		Marshaller m = c1.createMarshaller();
		Unmarshaller u = c1.createUnmarshaller();
		m.marshal(r, test1);
		Root r2 = (Root) u.unmarshal(test1);
		System.out.println("Test 1 =================");
		r2.printOut(System.out);
		System.out.println("========================");
	}
	
	@Test
	public void test2() throws JAXBException {
		Root r = createRoot();
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext2();
		Marshaller m = c2.createMarshaller();
		Unmarshaller u = c2.createUnmarshaller();
		m.marshal(r, test1);
		Root r2 = (Root) u.unmarshal(test1);
		System.out.println("Test 2 =================");
		r2.printOut(System.out);
		System.out.println("========================");
	}
	
	@Test
	public void test3() throws JAXBException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext1();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext2();
		Unmarshaller u = c2.createUnmarshaller();
		Root r2 = (Root) u.unmarshal(test1);
		System.out.println("Test 3 =================");
		r2.printOut(System.out);
		System.out.println("========================");
	}
	
	@Test
	public void test4() throws JAXBException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext1();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext2();
		Unmarshaller u = c2.createUnmarshaller();
		Root r2 = (Root) u.unmarshal(test1);
		
		System.out.println("Test 4 =================");
		r2.printOut(System.out);
		
		String xPath = "children[1]/Leaf/name/text()";
		String name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);
		
		xPath = "//children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);

		xPath = "./children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println(xPath + " ***unexpectedly*** gives " + name);

		xPath = "/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);

		xPath = "/Root/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);
		
		System.out.println("========================");
	}
	
	@Test
	public void test5() throws JAXBException {
		DummyEnvelope dummy;
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext2();
		{
			Root r = createRoot();
			JAXBContext c1 = createJAXBContext1();
			Marshaller m = c1.createMarshaller();
			m.marshal(r, test1);
			c2 = createJAXBContext2();
			Unmarshaller u = c2.createUnmarshaller();
			Root r2 = (Root) u.unmarshal(test1);
			
			System.out.println("Test 5 =================");
			r2.printOut(System.out);
			
			dummy = new DummyEnvelope();
			dummy.root = r2;
		}
		
		
		
		String xPath = "//children[1]/Leaf/name/text()";
		String name = c2.getValueByXPath(dummy, xPath, null, String.class);
		System.out.println(xPath + " ***unexpectedly*** gives " + name);

		xPath = "/Root/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(dummy, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);
		
		
		System.out.println("========================");
	}
	
	
	@Test
	public void test6() throws JAXBException, ParserConfigurationException, SAXException, IOException, XPathExpressionException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext1();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		
		DocumentBuilder b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document d = b.parse(test1);
		XPath xp = XPathFactory.newInstance().newXPath();
		
		
		System.out.println("Test 6 =================");

		String xPath = "//children[1]/Leaf/name/text()";
		XPathExpression xpr = xp.compile(xPath);
		String name = xpr.evaluate(d);
		System.out.println(xPath + " gives " + name);

		xPath =  "/Root/children[1]/Leaf/name/text()";
		xpr = xp.compile(xPath);
		name = xpr.evaluate(d);
		System.out.println(xPath + " gives " + name);
	
	
		System.out.println("========================");
	}

	private Root createRoot() {
		Root r = new Root();
		r.name = "RootA";
		for (int i = 1; i <= 10; i++) {
			Child c = new Child();
			c.name = "A" + i;
			Leaf l = new Leaf();
			l.name = "a" + i;
			c.leaf = l;
			r.addChild(c);
		}
		return r;
	}
	
	private Root loadRoot(File f) throws JAXBException {
		JAXBContext c1 = createJAXBContext1();
		Unmarshaller u = c1.createUnmarshaller();
		return (Root) u.unmarshal(f);
	}
	
	private JAXBContext createJAXBContext1() throws JAXBException {
		return JAXBContext.newInstance("uk.co.his.test.jaxb.model");
	}
	
	private org.eclipse.persistence.jaxb.JAXBContext createJAXBContext2() throws JAXBException {
		return (org.eclipse.persistence.jaxb.JAXBContext) org.eclipse.persistence.jaxb.JAXBContextFactory.createContext("uk.co.his.test.jaxb.model", null);
	}
	
	
}
