package uk.co.his.test.jaxb.junit;

import java.io.File;
import java.io.IOException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.junit.Test;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import uk.co.his.test.jaxb.model.Child;
import uk.co.his.test.jaxb.model.DummyEnvelope;
import uk.co.his.test.jaxb.model.Leaf;
import uk.co.his.test.jaxb.model.Root;

public class TestMoxy2 {
	
	public static final File test1 = new File("test-output/Test1.xml");

	
	
	@Test
	public void testXPathAndMoxy() throws JAXBException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext();
		Unmarshaller u = c2.createUnmarshaller();
		Root r2 = (Root) u.unmarshal(test1);
		
		System.out.println("testXPathAndMoxy =================");
		r2.printOut(System.out);
		
		//The Current context is the Root object, so 'children' is any child element named 'children'.
		String xPath = "children[1]/Leaf/name/text()";
		String name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("a) " + xPath + " gives " + name);
		

		// '//' means any descendants of the Document Root so //children means any 'children' elements in the current 'document'
		xPath = "//children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("b) " + xPath + " gives " + name);

		// '.' means the current context node so './children' is the same as 'children', and should give the same result as (a)
		//Incompatibility 1?
		xPath = "./children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("c) " + xPath + " ***unexpectedly*** gives " + name);
		
		xPath = "children[1]";
		Child child = c2.getValueByXPath(r2, xPath, null, Child.class);
		xPath = "//Leaf";
		Object result = c2.getValueByXPath(child, xPath, null, Object.class);
		System.out.println("d1) " + xPath + " ***unexpectedly*** gives " + result);
		
		xPath = "//Leaf";
		result = c2.getValueByXPath(r2, xPath, null, Object.class);
		System.out.println("d2) " + xPath + " ***unexpectedly*** gives " + result);
		
		xPath = "/name/text()";
		result = c2.getValueByXPath(child, xPath, null, Object.class);
		System.out.println("d3) " + xPath + " ***unexpectedly*** gives " + result);

		// '/' means the document element root, is the 'invisible' root or the top element in the POJO containment hierarchy
		xPath = "/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("e) " + xPath + " gives " + name);

		// '/Root' means the Root child of the document element root.
		// I take it therefore there is no invisible Document Element Root, and that '/' == the topmost object in the POJO containment hierarchy
		xPath = "/Root/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("f) " + xPath + " gives " + name);
		
		System.out.println("========================");
	}
	
	@Test
	public void testXPathAndMoxy2() throws JAXBException {
		DummyEnvelope dummy;
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext();
		{
			Root r = createRoot();
			JAXBContext c1 = createJAXBContext();
			Marshaller m = c1.createMarshaller();
			m.marshal(r, test1);
			
			Unmarshaller u = c2.createUnmarshaller();
			Root r2 = (Root) u.unmarshal(test1);
			
			System.out.println("testXPathAndMoxy2 =================");
			r2.printOut(System.out);
			
			dummy = new DummyEnvelope();
			dummy.root = r2;
		}
		
		// '//' means any descendants of the Document Root so //children means any 'children' elements in the current 'document'
		// So regardless of whether DummyEnvelope is the root or Root is this should work
		//Incompatibility 2
		String xPath = "//children[1]/Leaf/name/text()";
		String name = c2.getValueByXPath(dummy, xPath, null, String.class);
		System.out.println(xPath + " ***unexpectedly*** gives " + name);

		//From testXPathAndMoxy '/' should be the topmost POJO in the containment hierarchy, so '/' is DummyEnvelope
		xPath = "/Root/children[1]/Leaf/name/text()";
		name = c2.getValueByXPath(dummy, xPath, null, String.class);
		System.out.println(xPath + " gives " + name);
		
		
		System.out.println("========================");
	}
	
	
	@Test
	public void testDomAndXPath() throws JAXBException, ParserConfigurationException, SAXException, IOException, XPathExpressionException {
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		
		DocumentBuilder b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document d = b.parse(test1);
		XPath xp = XPathFactory.newInstance().newXPath();
		
		
		System.out.println("testDomAndXPath =================");
		
		//The current context '.' is the 'invisible' Document Element Root
		String xPath = "./Root/children[1]/Leaf/name/text()";
		XPathExpression xpr = xp.compile(xPath);
		String name = xpr.evaluate(d);
		System.out.println(xPath + " gives \"" + name +"\"");
		
		//'Root' is a child of the Document Element Root so this works as well
		xPath = "Root/children[1]/Leaf/name/text()";
		xpr = xp.compile(xPath);
		name = xpr.evaluate(d);
		System.out.println(xPath + " gives \"" + name +"\"");

		
		// '//' specifies any path from the Root so this is fine 
		xPath = "//children[1]/Leaf/name/text()";
		xpr = xp.compile(xPath);
		name = xpr.evaluate(d);
		System.out.println(xPath + " gives " + name);

		// '/' specifies the 'invisible' Document Element Root
		xPath =  "/Root/children[1]/Leaf/name/text()";
		xpr = xp.compile(xPath);
		name = xpr.evaluate(d);
		System.out.println(xPath + " gives " + name);
	
	
		System.out.println("========================");
	}

	private Root createRoot() {
		Root r = new Root();
		r.name = "RootA";
		for (int i = 1; i <= 10; i++) {
			Child c = new Child();
			c.name = "A" + i;
			Leaf l = new Leaf();
			l.name = "a" + i;
			c.leaf = l;
			r.addChild(c);
		}
		return r;
	}
	
	private org.eclipse.persistence.jaxb.JAXBContext createJAXBContext() throws JAXBException {
		return (org.eclipse.persistence.jaxb.JAXBContext) org.eclipse.persistence.jaxb.JAXBContextFactory.createContext("uk.co.his.test.jaxb.model", null);
	}
	
	
}
