package uk.co.his.test.jaxb.junit;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import uk.co.his.test.jaxb.model.Child;
import uk.co.his.test.jaxb.model.Leaf;
import uk.co.his.test.jaxb.model.Root;

public class TestMoxy3 {
	
	public static final File test1 = new File("test-output/Test1.xml");

	
	@BeforeClass
	public static void boostrap() throws JAXBException, IOException, SAXException, TransformerException {
		createFile(test1);
	}
	
	@Test
	public void testXPathAndMoxy() throws JAXBException, SAXException, IOException, ParserConfigurationException, XPathExpressionException {
		
		org.eclipse.persistence.jaxb.JAXBContext c2 = createJAXBContext();
		Unmarshaller u = c2.createUnmarshaller();
		Root r2 = (Root) u.unmarshal(test1);
		
		DocumentBuilder b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document d = b.parse(test1);
		XPath xp = XPathFactory.newInstance().newXPath();
		
		
		System.out.println("========= TESTS ===============");
		
		//Edge case (1) The current context in Moxy is already the instance of Root
		String xPath = "Root/children[1]/Leaf/name/text()";
		XPathExpression xpr = xp.compile(xPath);
		String domResultStr = xpr.evaluate(d);
		String jaxbResultStr = c2.getValueByXPath(r2, xPath, null, String.class);
		System.out.println("1) " + xPath + " gives \"" + domResultStr + "\" using DOM and \""+ jaxbResultStr + "\" using Moxy");
		
		//Edge case (2) Xpath expressions using javax.xml.xpath and DOM have a global scope regardless of current context
		//First get some child somewhere in the middle of the hierarchy
		String xPathA = "//children[1]";
		xpr = xp.compile(xPathA);
		Node domResultNode = (Node) xpr.evaluate(d, XPathConstants.NODE);
		Child jaxbResultChild = c2.getValueByXPath(r2, xPathA, null, Child.class);
		
		//Now find out whether we can navigate 'back up' the hierachy
		xPath = "//children[2]/name/text()";
		Assert.assertTrue(xPathA + " gave unexpected null result for javax.xml.xpath and DOM", domResultNode != null);
		xpr = xp.compile(xPath);
		domResultStr = xpr.evaluate(domResultNode);
		
		Assert.assertTrue(xPathA + " gave unexpected null result for MOXy", jaxbResultChild != null);
		jaxbResultStr = c2.getValueByXPath(jaxbResultChild, xPath, null, String.class);
		System.out.println("2) " + xPath + " via " + xPathA + " gives \"" + domResultStr + "\" using DOM and \""+ jaxbResultStr + "\" using Moxy");
		
		//Edge case (3) '/' means the Document Element Root in XPath, but evaluates to the current node in Moxy
		xPath = "/name/text()";
		xpr = xp.compile(xPath);
		domResultStr = xpr.evaluate(domResultNode);
		jaxbResultStr = c2.getValueByXPath(jaxbResultChild, xPath, null, String.class);
		System.out.println("3) " + xPath + " via " + xPathA + " gives \"" + domResultStr + "\" using DOM and \""+ jaxbResultStr + "\" using Moxy");
		

		//BUG (4) '.' means the current context in XPath, but not in MOXy
		xPath = "./name/text()";
		xpr = xp.compile(xPath);
		domResultStr = xpr.evaluate(domResultNode);
		jaxbResultStr = c2.getValueByXPath(jaxbResultChild, xPath, null, String.class);
		System.out.println("4) " + xPath + " via " + xPathA + " gives \"" + domResultStr + "\" using DOM and \""+ jaxbResultStr + "\" using Moxy");
		
		/* This works
		xPath = "name/text()";
		xpr = xp.compile(xPath);
		domResultStr = xpr.evaluate(domResultNode);
		jaxbResultStr = c2.getValueByXPath(jaxbResultChild, xPath, null, String.class);
		System.out.println("4b) " + xPath + " via " + xPathA + " gives \"" + domResultStr + "\" using DOM and \""+ jaxbResultStr + "\" using Moxy");
		*/
	}
	
	
	private static void createFile(File named) throws JAXBException, IOException, SAXException, TransformerException {
		
		Root r = createRoot();
		JAXBContext c1 = createJAXBContext();
		Marshaller m = c1.createMarshaller();
		m.marshal(r, test1);
		System.out.println("===== Here is the target Object Hierachy =======");
		r.printOut(System.out);
		System.out.println("===== Here is resultant XML file contents =======");
		echoFile(test1, System.out);
	}

	private static Root createRoot() {
		Root r = new Root();
		r.name = "RootA";
		for (int i = 1; i <= 10; i++) {
			Child c = new Child();
			c.name = "A" + i;
			Leaf l = new Leaf();
			l.name = "a" + i;
			c.leaf = l;
			r.addChild(c);
		}
		return r;
	}
	
	
	public static void echoFile(File which, PrintStream out) throws SAXException, IOException, TransformerException
	{
		TransformerFactory transFactory = TransformerFactory.newInstance();
		try
		{
			transFactory.setAttribute("indent-number", new Integer(2));
		}
		catch (IllegalArgumentException ignored) //If whatever we pick up doesn't support it, just carry on
		{ 
			System.err.println("Default TransformerFactory " + transFactory.getClass() + " does not support indent-number"); 
		} 
		Transformer transformer = transFactory.newTransformer();
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		try
		{
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
		}
		catch (IllegalArgumentException ignored) //If whatever we pick up doesn't support it, just carry on
		{ 
			System.err.println("Default TransformerFactory " + transFactory.getClass() + " does not support {http://xml.apache.org/xslt}indent-amount"); 
		} 
		try
		{
			transformer.transform(new StreamSource(which), new StreamResult(out));
		}
		catch (TransformerException e)
		{
			throw e;
		}
	}
	
	private static org.eclipse.persistence.jaxb.JAXBContext createJAXBContext() throws JAXBException {
		return (org.eclipse.persistence.jaxb.JAXBContext) org.eclipse.persistence.jaxb.JAXBContextFactory.createContext("uk.co.his.test.jaxb.model", null);
	}
	
	
}
